﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;

namespace WpfCustomControlSample
{
    /// <summary>
    /// Follow steps 1a or 1b and then 2 to use this custom control in a XAML file.
    ///
    /// Step 1a) Using this custom control in a XAML file that exists in the current project.
    /// Add this XmlNamespace attribute to the root element of the markup file where it is 
    /// to be used:
    ///
    ///     xmlns:MyNamespace="clr-namespace:WpfCustomControlSample"
    ///
    ///
    /// Step 1b) Using this custom control in a XAML file that exists in a different project.
    /// Add this XmlNamespace attribute to the root element of the markup file where it is 
    /// to be used:
    ///
    ///     xmlns:MyNamespace="clr-namespace:WpfCustomControlSample;assembly=WpfCustomControlSample"
    ///
    /// You will also need to add a project reference from the project where the XAML file lives
    /// to this project and Rebuild to avoid compilation errors:
    ///
    ///     Right click on the target project in the Solution Explorer and
    ///     "Add Reference"->"Projects"->[Select this project]
    ///
    ///
    /// Step 2)
    /// Go ahead and use your control in the XAML file.
    ///
    ///     <MyNamespace:CustomControl1/>
    ///
    /// </summary>
    /// 
    [DefaultProperty("Value")]
    public class SampleCustomControl : Control
    {
        public static readonly DependencyProperty ValueProperty;        
        public static readonly DependencyProperty CornerRadiusProperty;       
		
        static SampleCustomControl()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(SampleCustomControl), new FrameworkPropertyMetadata(typeof(SampleCustomControl)));

            FrameworkPropertyMetadata frameworkPropertyMetadata = new FrameworkPropertyMetadata("0", FrameworkPropertyMetadataOptions.AffectsRender | FrameworkPropertyMetadataOptions.BindsTwoWayByDefault);
            ValueProperty = DependencyProperty.Register("Value", typeof(string), typeof(SampleCustomControl), frameworkPropertyMetadata);
            frameworkPropertyMetadata = new FrameworkPropertyMetadata(new CornerRadius(0, 0, 0, 0));
            CornerRadiusProperty = Border.CornerRadiusProperty.AddOwner(typeof(SampleCustomControl), frameworkPropertyMetadata);            
        }

        public CornerRadius CornerRadius
        {
            get { return (CornerRadius)GetValue(CornerRadiusProperty); }
            set { SetValue(CornerRadiusProperty, value); }
        }

        public string Value
        {
            get { return (string)GetValue(ValueProperty); }
            set { SetValue(ValueProperty, value); }
        }
    }
}
