﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;

namespace WpfUserControlSample
{
    /// <summary>
    /// Interaction logic for UserControl1.xaml
    /// </summary>    
    [DefaultProperty("Value")]
    public partial class SampleUserControl : UserControl
    {
        public static readonly DependencyProperty ValueProperty;
        public static readonly RoutedEvent ValueChangedEvent;
        static SampleUserControl()
        {
            FrameworkPropertyMetadata frameworkPropertyMetadata = new FrameworkPropertyMetadata("0", FrameworkPropertyMetadataOptions.Journal | FrameworkPropertyMetadataOptions.BindsTwoWayByDefault, new PropertyChangedCallback(SampleUserControl.OnValuePropertyChanged));
            ValueProperty = DependencyProperty.Register("Value", typeof(object), typeof(SampleUserControl), frameworkPropertyMetadata);

            ValueChangedEvent = EventManager.RegisterRoutedEvent("ValueChanged", RoutingStrategy.Bubble, typeof(RoutedPropertyChangedEventHandler<object>), typeof(SampleUserControl));
        }

        public SampleUserControl()
        {
            InitializeComponent();
        }

        public object Value
        {
            get { return GetValue(ValueProperty); }
            set { SetValue(ValueProperty, value); }
        }

        private void OnButtonClick(object sender, RoutedEventArgs e)
        {
            Value = "0";
        }

        private static void OnValuePropertyChanged(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            SampleUserControl userControl = (SampleUserControl)dependencyObject;
            if (userControl == null)
                return;
            
            userControl.FireValueChanged(e.OldValue, e.NewValue);
        }

        public void FireValueChanged(object oldValue, object newValue)
        {
            RoutedPropertyChangedEventArgs<object> e = new RoutedPropertyChangedEventArgs<object>(oldValue, newValue);
            e.RoutedEvent = ValueChangedEvent;
            base.RaiseEvent(e);
        }

        public event RoutedPropertyChangedEventHandler<object> ValueChanged
        {
            add
            {
                base.AddHandler(ValueChangedEvent, value);
            }
            remove
            {
                base.RemoveHandler(ValueChangedEvent, value);               
            }
        }
        
    }
}
